
-- SQL Server 2000
SELECT ProductID, ProductName
FROM Products Product
FOR XML AUTO


-- The OPENXML function performs the opposite function to the FOR XML clause by creating a rowset from an XML document, as shown in the following example.
DECLARE @doc nvarchar(1000)
SET @doc = '<Order OrderID = "1011">
			<Item ProductID="1" Quantity="2"/>
			<Item ProductID="2" Quantity="1"/>
		</Order>'
DECLARE @xmlDoc integer
EXEC sp_xml_preparedocument @xmlDoc OUTPUT, @doc
SELECT * FROM
OPENXML (@xmlDoc, 'Order/Item', 1)
WITH
(OrderID integer '../@OrderID',
 ProductID integer,
 Quantity integer)
EXEC sp_xml_removedocument @xmlDoc

-- SQL Server 2005
SELECT ProductID AS '@ProductID',
	ProductName AS 'ProductName'
FROM Products
FOR XML PATH ('Product'), ROOT ('Products')


-- In addition to enhancing the existing XML features that had been introduced in SQL Server 2000, SQL Server 2005 added a new, native xml data type that enables you to create variables and columns for XML data, as shown in the following example.
CREATE TABLE SalesOrders
(OrderID integer PRIMARY KEY,
 OrderDate datetime,
 CustomerID integer,
 OrderNotes xml)

-- You can use the xml data type to store markup documents or semi-structured data in the database. Columns and variables can be used for untyped XML or typed XML, the latter of which is validated against an XML Schema Definition (XSD) schema. To define the schemas for data validation, developers can use the CREATE XML SCHEMA COLLECTION statement, as shown in the following example.
CREATE XML SCHEMA COLLECTION ProductSchema AS
'<?xml version="1.0" encoding="UTF-16"?>
<xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema">
  <!-- schema declarations go here -->
</xs:schema>'

-- After creating a schema collection, you can associate an xml variable or column with the schema declarations it contains by referencing the schema collection as shown in the following example.
CREATE TABLE SalesOrders
(OrderID integer PRIMARY KEY,
 OrderDate datetime,
 CustomerID integer,
 OrderNotes xml(ProductSchema))

-- Typed XML is validated against the declarations in the associated schema collection when values are inserted or updated, which makes it possible to enforce business rules about the structure of XML data for compliance or compatibility reasons.
-- The xml data type also provides a number of methods, which you can use to query and manipulate the XML data in an instance. For example, you can use the query method to query the XML in an instance of the xml data type, as shown in the following example.
declare @x xml
set @x=
'<Invoices>
<Invoice>
		<Customer>Kim Abercrombie</Customer>
		<Items>
			<Item ProductID="2" Price="1.99" Quantity="1" />
			<Item ProductID="3" Price="2.99" Quantity="2" />
			<Item ProductID="5" Price="1.99" Quantity="1" />
		</Items>
</Invoice>
	<Invoice>
		<Customer>Margaret Smith</Customer>
		<Items>
			<Item ProductID="2" Price="1.99" Quantity="1"/>
		</Items>
	</Invoice>
</Invoices>'
SELECT @x.query(
'<CustomerList>
{
for $invoice in /Invoices/Invoice
return $invoice/Customer
}
</CustomerList>')

-- The query in this example uses an XQuery expression that finds each Invoice element in the document and returns an XML document that contains the Customer element from each Invoice element, as shown in the following example.
<CustomerList>
  <Customer>Kim Abercrombie</Customer>
  <Customer>Margaret Smith</Customer>
</CustomerList>

-- Another significant XML-related feature that was introduced in SQL Server 2005 is support for XML indexes. 
CREATE PRIMARY XML INDEX idx_xml_Notes
ON SalesOrders (Notes)
GO

CREATE XML INDEX idx_xml_Path_Notes
ON SalesOrders (Notes)
USING XML INDEX idx_xml_Notes
FOR PATH
GO

-- SQL Server 2008 Additions

-- XML Schemas support wildcard sections in XML documents through the any, anyAttribute, and anyType declarations. For example, consider the following XML schema declaration.
<xs:complexType name="Order" mixed="true">
  <xs:sequence>
    <xs:element name="CustomerName"/>
    <xs:element name="OrderTotal"/>
    <xs:any namespace="##other" processContents="skip" 
minOccurs="0" maxOccurs="unbounded"/>
  </xs:sequence>
</xs:complexType>

-- This schema declaration defines an XML element named Order, which must contain sub-elements named CustomerName and OrderTotal. Additionally, the element can contain an unlimited number of other elements that belong to a different namespace than the one to which the Order type belongs. The following XML shows an XML document that contains an instance of an Order element as defined by this schema declaration. Note that the order also contains a shp:Delivery element, which is not explicitly defined in the schema.
<Invoice xmlns="http://adventure-works.com/order"
	xmlns:shp="http://adventure-works.com/shipping">
  <Order>
    <CustomerName>Graeme Malcolm</CustomerName>
    <OrderTotal>299.99</OrderTotal>
    <shp:Delivery>Express</shp:Delivery>
  </Order>
</Invoice>

-- For example, you can use a list type to define the valid sizes for a product as shown in the following example.
<xs:simpleType name="sizeListType">
  <xs:list>
    <xs:simpleType>
      <xs:restriction base="xs:string">
        <xs:enumeration value="S"/>
        <xs:enumeration value="M"/>
        <xs:enumeration value="L"/>
      </xs:restriction>
    </xs:simpleType>
  </xs:list>
</xs:simpleType>

/*
This schema declaration enables you to create an element that lists all of the sizes in which a product can be purchased as a list of values separated by white space, as shown in the following example:
<AvailableSizes>S M L</AvailableSizes>

However, what if you want to support two different ways to express the size of a product? For example, suppose a cycling equipment retailer sells cycling clothes in small, medium, and large sizes, but also sells bicycles in numerical sizes relating to the frame size (such as 18, 20, 22, and 24)? To enable you to accomplish this, SQL Server 2008 adds support for union types that contain list types, which you can use to merge multiple list type definitions and restrictions into a single type. For example, the following Transact-SQL code creates an XML schema collection that defines a productSizeType type in which valid values include a list of numeric sizes (18, 20, 22, and 24) and a list of named sizes (S, M, and L).
*/

CREATE XML SCHEMA COLLECTION CatalogSizeSchema AS
N'<?xml version="1.0" encoding="UTF-16"?>
<xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema">
	<xs:simpleType name="productSizeType">
		<xs:union>
			<xs:simpleType>
			  <xs:list>
				<xs:simpleType>
				  <xs:restriction base="xs:integer">
					<xs:enumeration value="18"/>
					<xs:enumeration value="20"/>
					<xs:enumeration value="22"/>
					<xs:enumeration value="24"/>
				  </xs:restriction>
				</xs:simpleType>
			  </xs:list>
			</xs:simpleType>
			<xs:simpleType>
			  <xs:list>
				<xs:simpleType>
				  <xs:restriction base="xs:string">
					<xs:enumeration value="S"/>
					<xs:enumeration value="M"/>
					<xs:enumeration value="L"/>
				  </xs:restriction>
				</xs:simpleType>
			  </xs:list>
			</xs:simpleType>
		</xs:union>
	</xs:simpleType>
</xs:schema>'

-- With this declaration in the schema, any elements based on the productSizeType can contain either kind of list; so both of the product elements in the following example would be valid instances of the productSizeType data type.
<Catalog>
  <Product>
    <ProductName>Road Bike</ProductName>
    <AvailableSizes>22 24</AvailableSizes>
  </Product>
  <Product>
    <ProductName>Cycling Jersey</ProductName>
    <AvailableSizes>S M L</AvailableSizes>
  </Product>
</Catalog>

-- SQL Server 2008 adds support for the let clause, which is used to assign values to variables in an XQuery expression such as the following example:
declare @x xml
set @x=
'<Invoices>
<Invoice>
		<Customer>Kim Abercrombie</Customer>
		<Items>
			<Item ProductID="2" Price="1.99" Quantity="1" />
			<Item ProductID="3" Price="2.99" Quantity="2" />
			<Item ProductID="5" Price="1.99" Quantity="1" />
		</Items>
</Invoice>
	<Invoice>
		<Customer>Margaret Smith</Customer>
		<Items>
			<Item ProductID="2" Price="1.99" Quantity="1"/>
		</Items>
	</Invoice>
</Invoices>'
SELECT @x.query(
'<Orders>
{
for $invoice in /Invoices/Invoice
let $count :=count($invoice/Items/Item)
order by $count
return
<Order>
{$invoice/Customer}
<ItemCount>{$count}</ItemCount>
</Order>
}
</Orders>')

-- This example returns the following XML.
<Orders>
	<Order>
		<Customer>Margaret Smith</Customer>
		<ItemCount>1</ItemCount>
	</Order>
	<Order>
		<Customer>Kim Abercrombie</Customer>
		<ItemCount>3</ItemCount>
	</Order>
</Orders>

-- SQL Server 2008 adds support for using an xml variable in an insert expression to insert XML data into an existing XML structure. For example, suppose an xml variable named @productList contains the following XML:
<Products>
  <Bike>Mountain Bike</Bike>
  <Bike>Road Bike</Bike>
</Products>

-- You could use the following code to insert a new bike into the product list:
DECLARE @newBike xml
SET @newBike = '<Bike>Racing Bike</Bike>'
@productList.modify
('insert sql:variable("@newBike") as last into (/Products)[1]')

/*
After running this code, the @productList variable would contain the following XML.
<Products>
  <Bike>Mountain Bike</Bike>
  <Bike>Road Bike</Bike>
  <Bike>Racing Bike</Bike>
</Products>
*/
